/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#include <string>

#include <triton/client/grpc_client.h>
#include <triton/client/http_client.h>

#include <condition_variable>

#define STREAM
#ifdef STREAM
#include <condition_variable>

#endif

namespace tc = triton::client;

using CompletionResultCallback =
    std::function<void(const std::string &, bool over)>;
using ResultList = std::vector<std::shared_ptr<tc::InferResult>>;

struct chatConfig {
    int request_type_data = 0;
    std::string request_id = "message_1";
    std::string chatPrompt = "message";
    float temperature_input = 0.0;
    int32_t temperature_input_data = 0.0;
    int32_t top_k_input_data = 40;
    float top_p_input_data = 0.95;
    int32_t n_keep_input_data = 0;
    int32_t n_predict_input_data = 512;
    bool cache_prompt_input_data = true;
    std::vector<std::string> stop_input_data{"<|im_end|>"};
    std::vector<float> lora_scale_input_data = {0, 0};
    bool stream_input_data = true;
};

class LLM {
public:
    LLM();
    // static void llm_callback(tc::InferResult* result);
    int setLoraScale(int task_id, std::vector<float> &lora_scale_input_data);
    std::string chat(const std::string &prompt, const int &task_id);
    void setCompletionResultCallback(CompletionResultCallback callback);
    std::string GetConfigValue(const char *key, const char *engine);
    bool setLlmModelName(const std::string &modelName);
    void stopChat();
    int getLoraCount();
    void llm_callback(tc::InferResult *result);
    bool llm_load_model();
    void modelInfoUpdate();
    ~LLM();
    std::mutex mutex_;
    std::condition_variable cv_;
    ResultList result_list_;
    std::string modelName_;
    std::string modelInfo_;
    int n_predict_input_;

private:
    std::string request_id_ = "";
    std::string default_model_info_ = "";
    void chat_inputs(struct chatConfig config,
                     std::vector<tc::InferInput *> &inputs);
    std::string modelVersion_ = "";

    CompletionResultCallback completionCallbacks_;

    std::string httpServerUrl_ = "http://localhost:8000";
    std::unique_ptr<tc::InferenceServerGrpcClient> loadClient_ = nullptr;

    std::string grpcServerUrl_ = "localhost:8001";
    std::unique_ptr<tc::InferenceServerGrpcClient> grpcClient_ = nullptr;
};